/* 
 * Copyright (c) 2002 The University of Wroclaw.
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Rev: 701 $ $LastChangedDate: 2002-08-30 01:09:52 +0200 (Fri, 30 Aug 2002) $
 *
 * Runtime environment for Gont, Std module interface.
 * 
 * This module is opened at the beginning of every compilation unit.
 */
 
/* Return decimal string representation of [i].  */
string itoa(int t) = "Std__itoa";

/* Convert [s] from decimal string to integer.  If [s] is not valid decimal
   integer with optional white spaces at the beginning and at the end
   Invalid_argument is raised.  This is different from atoi(3), 
   for example in C [atoi("3ble") == 3], in Gont exception is raised.  */
int atoi(string s) = "Std__atoi";

/* Return decimal string representation of [f].  */
string ftoa(float f) = "Std__ftoa";

/* Convert [s] from decimal string to float.  If [s] is not valid floating
   point number (as with strtod(3)), with optional white spaces at the i
   beginning and at the end Invalid_argument is raised.  */
float atof(string) = "Std__atof";

/* Return either "true" iff [b == true] or "false" otherwise.  */
string string_of_bool(bool b);

/* Return [true] iff [s == "true"], [false] iff [s == "false"], raise
   Invalid_argument otherwise.  */
bool bool_of_string(string s);

/* An alias for Std::itoa.  */
string int_of_string(int) = "Std__itoa";

/* An alias for Std::atoi.  */
int string_of_int(string) = "Std__atoi";

/* An alias for Std::ftoa.  */
string string_of_float(float) = "Std__ftoa";

/* An alias for Std::atof.  */
float float_of_string(string) = "Std__atof";

/* [f] is ignored.  It is useful if you want discard return value of function,
   and don't want warning, then you do: [ignore(f())].  Of course this is
   only relevant if you compile with -Wignored-retval.  */
void ignore('a f);

/* Return [f] partially evaluated on [c], i.e. [fun (x) f(c, x)].  */
*('a ('b)) partial(*('a ('c, 'b)) f, 'c c);

/* Return [f] partially evaluated on [c, d], i.e. [fun (x) f(c, d, x)].  */
*('a ('b)) partial2(*('a ('c, 'd, 'b)) f, 'c c, 'd d);

/* Print [s] on [stdout].  */
void print_string(string s) = "Std__print_string";

/* Print string representation of [i] on [stdout].  */
void print_int(int i);

/* Print string representation of [f] on [stdout].  */
void print_float(float f);

/* Print [s + "\n"] on [stdout].  */
void print_endline(string s);

/* Print ["\n"] on [stdout].  */
void print_newline();

/* An alias for Std::print_string.  Note that it doesn't append ["\n"].  */
void puts(string s) = "Std__print_string";

/* Print [s] on [stderr].  */
void prerr_string(string s) = "Std__prerr_string";

/* Print string representation of [i] on [stderr].  */
void prerr_int(int i);

/* Print string representation of [f] on [stderr].  */
void prerr_float(float f);

/* Print [s + "\n"] on [stderr].  */
void prerr_endline(string s);

/* Print ["\n"] on [stderr].  */
void prerr_newline();

/* Read and return one line of input from [stdin].  */
string read_line() = "Std__read_line";

/* Read one word (sequence of non-white-space characters) from [stdin],
   and return it converted to int.  It's more or less the same as 
   [scanf("%i", &i);] in C.  */
int read_int();

/* Read one word (sequence of non-white-space characters) from [stdin],
   and return it converted to int.  It's more or less the same as 
   [scanf("%f", &f);] in C.  */
float read_float();

/* Read one word (sequence of non-white-space characters) from [stdin],
   and return it.  It's more or less the same as [scanf("%s", buf);] in C.  */
string read_string() = "Std__read_string";

/* Return first element of pair [p].  */
'a fst(*('a, 'b) p);

/* Return second element of pair [p].  */
'b snd(*('a, 'b) p);

/* Type of characters.  */
type char;

/* Return character with ASCII code [x].  
   Raise Invalid_argument("Std::char_of_int") if [x] is outside allowed range 
   (0 - 255).
   UNSTABLE: We might do s/ASCII/UNICODE/. */
char char_of_int(int x);

/* Return ASCII code of [x].  
   UNSTABLE: We might do s/ASCII/UNICODE/. */
int int_of_char(char x);

/* Type of immutable, ML-style lists.  See List module for operations on 
   lists.  */
union <'a>list {
    void Nil;
    *('a, <'a>list) Cons;
}

/* Type of mutable references.  */
struct <'a>ref {
    'a ref_contents;
}

/* Get value from reference [r].  */
'a ref_get(<'a>ref r);

/* Set value in reference [r] to [a].  */
void ref_set(<'a>ref r, 'a a);

/* Option type.  */
union <'a>option {
	void None;
	'a Some;
}

/* Standard exceptions.  */

/* Raised when none of pattern clauses matched expression.  */
exception void Match_failure

/* Raised upon attempt to access [opt_struct] that is [null].  */
exception void Null_access

/* Raised by various library functions, when they are given arguments they
   can't handle.  */
exception string Invalid_argument

/* Raised by library functions, that are undefined for given arguments.  */
exception string Failure

/* Raised by library functions, when object to be returned was not to be 
   found.  */
exception void Not_found

/* Raised by library functions that had problems with underlaying C functions, 
   This generally involves [strerror(errno)].  */
exception string Sys_error

/* Raised by input functions, when end of file is reached.  */
exception void End_of_file

// vim:sts=4:sw=4
